#!/bin/bash

ACTION="$1"
shift

BASE_DIR=$(cd `dirname $(readlink -f $0)`;pwd -P)
HOME_DIR=$BASE_DIR/..
JAVA=$JAVA_HOME/bin/java
PID_FILE=$HOME_DIR/gamelet.pid
LOG_FILE=$HOME_DIR/gamelet.out
JAR_FILE=$HOME_DIR/lib/gamelet-0.0.1-SNAPSHOT.jar

export SW_AGENT_NAME=gamelet
export SW_AGENT_COLLECTOR_BACKEND_SERVICES=10.10.128.2:11800

STOP_WAIT_TIME="60"

if [ "/bin/java" == "$JAVA" ]; then
	JAVA="/opt/jdk8/bin/java"
	if [ ! -f "$JAVA" ]; then
		JAVA=`which java 2>/dev/null`
	fi
fi

SPRING_OPTS="-Dloader.path=$HOME_DIR/config/ -Dspring.config.location=$HOME_DIR/config/"
JAVA_OPTS="-server -Xms512m -Xmx4G -XX:MaxMetaspaceSize=256M -XX:MaxDirectMemorySize=2G -XX:+UseParNewGC -XX:+UseConcMarkSweepGC -XX:+UseCMSCompactAtFullCollection -XX:CMSFullGCsBeforeCompaction=0 -XX:+CMSClassUnloadingEnabled -XX:+CMSParallelRemarkEnabled  -XX:CMSInitiatingOccupancyFraction=80 -XX:+PrintClassHistogram -XX:+PrintGCDetails -XX:+PrintGCTimeStamps -XX:+PrintHeapAtGC -Xloggc:$HOME_DIR/gc.log -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=$HOME_DIR/$SW_AGENT_NAME.hprof"
JDWP_OPTS="-agentlib:jdwp=transport=dt_socket,address=0,suspend=n,server=y"
JAGENT_OPTS="-Djava.net.preferIPv4Stack=true -javaagent:${HOME_DIR}/agent/skywalking-agent.jar"

arguments=(-Dsun.misc.URLClassPath.disableJarChecking=true $JAGENT_OPTS $JAVA_OPTS $SPRING_OPTS $JDWP_OPTS -jar "$JAR_FILE" $RUN_ARGS "$@")

echo "$JAVA ${arguments[@]}"

echoRed() { echo $'\e[0;31m'"$1"$'\e[0m'; }
echoGreen() { echo $'\e[0;32m'"$1"$'\e[0m'; }
echoYellow() { echo $'\e[0;33m'"$1"$'\e[0m'; }

isRunning() {
  	ps -p "$1" &> /dev/null
}

do_start() {
	nohup "$JAVA" "${arguments[@]}" > "$LOG_FILE" 2>&1 &
    pid=$!
    echo "$pid" > "$PID_FILE"
  	[[ -z $pid ]] && { echoRed "Failed to start"; return 1; }
  	echoGreen "Started [$pid]"
}

start() {
  	if [ -f "$PID_FILE" ]; then
    	pid=$(cat "$PID_FILE")
    	isRunning "$pid" && { echoYellow "Already running [$PID_FILE]"; return 0; }
  	fi
  	do_start "$@"
}

do_stop() {
  	kill "$1" &> /dev/null || { echoRed "Unable to kill process $1"; return 1; }
  	for i in $(seq 1 $STOP_WAIT_TIME); do
    	isRunning "$1" || { echoGreen "Stopped [$1]"; rm -f "$2"; return 0; }
    	[[ $i -eq STOP_WAIT_TIME/2 ]] && kill "$1" &> /dev/null
    	sleep 1
  	done
  	echoRed "Unable to kill process $1";
  	return 1;
}

stop() {
  	[[ -f $PID_FILE ]] || { echoYellow "Not running (pidfile not found)"; return 0; }
  	pid=$(cat "$PID_FILE")
  	isRunning "$pid" || { echoYellow "Not running (process ${pid}). Removing stale pid file."; rm -f "$PID_FILE"; return 0; }
  	do_stop "$pid" "$PID_FILE"
}

do_force_stop() {
  	kill -9 "$1" &> /dev/null || { echoRed "Unable to kill process $1"; return 1; }
  	for i in $(seq 1 $STOP_WAIT_TIME); do
    	isRunning "$1" || { echoGreen "Stopped [$1]"; rm -f "$2"; return 0; }
    	[[ $i -eq STOP_WAIT_TIME/2 ]] && kill -9 "$1" &> /dev/null
    	sleep 1
 	done
  	echoRed "Unable to kill process $1";
  	return 1;
}

force_stop() {
  	[[ -f $PID_FILE ]] || { echoYellow "Not running (pidfile not found)"; return 0; }
  	pid=$(cat "$PID_FILE")
  	isRunning "$pid" || { echoYellow "Not running (process ${pid}). Removing stale pid file."; rm -f "$PID_FILE"; return 0; }
  	do_force_stop "$pid" "$PID_FILE"
}

restart() {
  	stop && start
}

force_reload() {
  	[[ -f $PID_FILE ]] || { echoRed "Not running (pidfile not found)"; return 7; }
  	pid=$(cat "$PID_FILE")
  	rm -f "$PID_FILE"
  	isRunning "$pid" || { echoRed "Not running (process ${pid} not found)"; return 7; }
  	do_stop "$pid" "$PID_FILE"
  	do_start
}

status() {
  	[[ -f "$PID_FILE" ]] || { echoRed "Not running"; return 3; }
  	pid=$(cat "$PID_FILE")
  	isRunning "$pid" || { echoRed "Not running (process ${pid} not found)"; return 1; }
  	echoGreen "Running [$pid]"
  	return 0
}

run() {
	"$JAVA" "${arguments[@]}"
  	result=$?
  	return "$result"
}

case "$ACTION" in
start)
  start "$@"; exit $?;;
stop)
  stop "$@"; exit $?;;
force-stop)
  force_stop "$@"; exit $?;;
restart)
  restart "$@"; exit $?;;
force-reload)
  force_reload "$@"; exit $?;;
status)
  status "$@"; exit $?;;
run)
  run "$@"; exit $?;;
*)
  echo "Usage: $0 {start|stop|force-stop|restart|force-reload|status|run}"; exit 1;
esac

exit 0